@@ -2,12 +2,11 @@
 This script display or export in JSON format the staus of the Cluster
 """
 
-import os, sys
+import sys
 import json
 import eventlet
 import socket
 import collections
-from numpy import std
 from urllib.parse import urlparse
 from oslo_config import cfg
 from swift.cli import recon
@@ -22,43 +21,44 @@ pool_size = 30
 pool = eventlet.GreenPool(pool_size)
 
 # Initialize script common oslo_config
-CLI_OPTS = [           
+CLI_OPTS = [
     cfg.BoolOpt('json', default=False, short='j',
-               help='Format output as json'),
+                help='Format output as json'),
     cfg.BoolOpt('all', default=False, short='a',
-               help='Active all options'),
+                help='Active all options'),
     cfg.BoolOpt('base2', default=False,
-               help='Print human-readable result in base 2 unit (kiB, MiB...)'),
+                help='Print human-readable result in base 2 unit (kiB, MiB...)'),
 ]
 
 # Initialize script cluster oslo_config
 CLI_OPTS_CLUSTER = [
     cfg.BoolOpt('status', default=False, short='s',
-               help='Show cluster state'),
+                help='Show cluster state'),
     cfg.BoolOpt('reason', default=False, short='r',
-               help='Show cluster state reason'),
+                help='Show cluster state reason'),
     cfg.BoolOpt('ntp_drift', default=False,
-               help='Show servers NTP drift'),
+                help='Show servers NTP drift'),
     cfg.BoolOpt('validate', default=False,
-               help='Validate server in the swift ring'),
+                help='Validate server in the swift ring'),
 ]
 
 # Initialize script disks oslo_config
 CLI_OPTS_DISK = [
     cfg.BoolOpt('usage', default=False, short='d',
-               help='Show disk usage for object ring'),
+                help='Show disk usage for object ring'),
     cfg.BoolOpt('unmounted', default=False, short='u',
-               help='Show unmounted or unknow state drives'),
+                help='Show unmounted or unknow state drives'),
     cfg.BoolOpt('filling', default=False, short='f',
-               help='Show disk filling for object ring'),
+                help='Show disk filling for object ring'),
     cfg.IntOpt('fmin', default=10,
                help='Display Node/device filling < DISK_FMIN'),
     cfg.IntOpt('fmax', default=90,
                help='Display Node/device filling > DISK_FMAX'),
     cfg.BoolOpt('deviation', default=False,
-               help='Show object ring deviation'),
+                help='Show object ring deviation'),
 ]
 
+
 def _prepare_config():
     """
     Prepare the oslo_config of scripts by analyse arguments
@@ -70,7 +70,8 @@ def _prepare_config():
     CONF.register_cli_opts(CLI_OPTS_DISK, group='disk')
     return CONF
 
-def size_suffix(size, base2 = False):
+
+def size_suffix(size, base2=False):
     """
     Format the disk size in human readable
     param size: the size in octet
@@ -78,7 +79,7 @@ def size_suffix(size, base2 = False):
     return: the formatted string
     """
     units = 'iB' if base2 else 'B'
-    suffixes = ['','k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y']
+    suffixes = ['', 'k', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y']
     kilo = 1024.0 if base2 else 1000.0
     for suffix in suffixes:
         if size < kilo:
@@ -86,10 +87,17 @@ def size_suffix(size, base2 = False):
         size = round(size / kilo, 2)
     return "%s %s%s" % (size, suffix, units)
 
+def std(array):
+    mean = sum(array) / len(array)
+    var = sum((l-mean)**2 for l in array) / len(array)
+    st_dev = var ** 0.5
+    return st_dev
+
+
 class Cluster():
     """Class representing the entire Cluster"""
 
-    def __init__(self,ring_names,args):
+    def __init__(self, ring_names, args):
         """
         Instance method to initialize Cluster
         param ring_names: a list with the différent ring to initialize
@@ -103,10 +111,11 @@ class Cluster():
         self.reason = ''
         self.disks_state = {}
         self.usage, self.usage_human = {}, {}
-        self.ntpdrift, self.validate  = {}, {}
-        self.filling, self.fmin, self.fmax, self.deviation = None, None, None, None
+        self.ntpdrift, self.validate = {}, {}
+        self.filling, self.fmin = None, None
+        self.fmax, self.deviation = None, None
         self.hosts_in_error = set()
-        self.region_with_state = {'unmounted':0,'unknown':0}
+        self.region_with_state = {'unmounted': 0, 'unknown': 0}
         self.disks_state_by_region = {}
         self.analyse()
 
@@ -115,7 +124,7 @@ class Cluster():
         Instance method to analyse Cluster
         """
         # True if one options is True
-        all_analyze = any( 
+        all_analyze = any(
                 (
                     self.args.all,
                     self.args.json,
@@ -136,29 +145,32 @@ class Cluster():
             )
         )
         if disks_analyze or all_analyze:
-            for name,ring in self.rings.items():
+            for name, ring in self.rings.items():
                 # Get properties of all rings
                 self.disks_state[name] = {}
                 self.disks_state[name]['unmounted'] = ring.disks_unmounted()
                 self.disks_state[name]['unknown'] = ring.disks_unknown()
                 self.usage[name] = ring.disks_usage()
-                self.usage_human[name] = ring.disks_usage_human(self.args.base2)
+                self.usage_human[name] = ring.disks_usage_human(
+                    self.args.base2)
                 self.ntpdrift[name] = ring.ntpdrift()
-                self.validate[name]= ring.validate()
-                for name_st,ds in self.disks_state[name].items():
+                self.validate[name] = ring.validate()
+                for name_st, ds in self.disks_state[name].items():
                     # Aggregate disks staus by region
-                    for region,value in ds.items():
-                        self.disks_state_by_region.setdefault(region,{})
-                        for host,devices in value.items():
-                            self.disks_state_by_region[region].setdefault(host,{})
-                            self.disks_state_by_region[region][host] = {name_st: devices}
+                    for region, value in ds.items():
+                        self.disks_state_by_region.setdefault(region, {})
+                        for host, devices in value.items():
+                            self.disks_state_by_region[region].\
+                                setdefault(host, {})
+                            self.disks_state_by_region[region][host] = {
+                                name_st: devices}
                 self.hosts_in_error.update(ring.hosts_in_error)
-            for region,hosts in self.disks_state_by_region.items():
+            for region, hosts in self.disks_state_by_region.items():
                 # Increment region with unmounted or unknown disk state
                 for st_name in self.region_with_state.keys():
-                    if len({k:v for (k,v) in hosts.items() if st_name in v}):
+                    if len({k: v for (k, v) in hosts.items() if st_name in v}):
                         self.region_with_state[st_name] += 1
-            for state,value in self.region_with_state.items():
+            for state, value in self.region_with_state.items():
                 if value > 1:
                     self.status = "Critical"
                     self.reason = f"{self.reason}{value} regions with {state} state drives\n"
@@ -169,11 +181,12 @@ class Cluster():
             if self.deviation > 5 and self.status == "OK":
                 self.status = "Warn"
                 self.reason = "Deviation higher than 5."
-        if self.status == "OK" and (all_analyze or self.args.cluster.ntp_drift):
+        if self.status == "OK" \
+                and (all_analyze or self.args.cluster.ntp_drift):
             if [n for n in self.ntpdrift.values() if n]:
                 self.status = "Warn"
                 self.reason = "Some servers have ntpdrift"
-        if self.status == "OK" and (all_analyze or self.args.cluster.validate): 
+        if self.status == "OK" and (all_analyze or self.args.cluster.validate):
             if [n for n in self.ntpdrift.values() if n]:
                 self.status = "Warn"
                 self.reason = "Some servers cannot be validate"
@@ -225,14 +238,17 @@ class Cluster():
         Instance method to display disks in unmounted or unknown state
         """
         rs = self.region_with_state
-        print(f"Region with umounted/unknown state drives: {rs['unmounted']}/{rs['unknown']}")
-        if self.region_with_state['unmounted'] or self.region_with_state['unknown']:
+        print(f"Region with umounted/unknown state drives: {rs['unmounted']}/ {rs['unknown']}")
+        if self.region_with_state['unmounted'] \
+                or self.region_with_state['unknown']:
             print("Unmounted/unknown state disks output per region:")
-            for region,nodes in self.disks_state_by_region.items():
+            for region, nodes in self.disks_state_by_region.items():
                 txt = ''
-                for node,v in sorted(nodes.items()):
-                    unmounted = ','.join(sorted(v['unmounted'] if 'unmounted' in v else []))
-                    unknown =  ','.join(sorted(v['unknown'] if 'unknown' in v else []))
+                for node, v in sorted(nodes.items()):
+                    unmounted = ','.join(sorted(v['unmounted']
+                                         if 'unmounted' in v else []))
+                    unknown = ','.join(sorted(v['unknown']
+                                              if 'unknown' in v else []))
                     txt = f"{txt}\t{node}: ({unmounted})/({unknown})\n"
                 if txt:
                     print(f"    {region}:{txt}")
@@ -243,7 +259,7 @@ class Cluster():
         Instance method to display the servers with Ntp drift in the cluster
         """
         print("Ntp drift:")
-        for name,ntp in self.ntpdrift.items():
+        for name, ntp in self.ntpdrift.items():
             print(f"  {name}: {ntp}")
         print('')
 
@@ -252,7 +268,7 @@ class Cluster():
         Instance method to display the invalidate servers in the cluster
         """
         print("Validate server output:")
-        for name,val in self.validate.items():
+        for name, val in self.validate.items():
             print(f"  {name}: {val}")
         print('')
 
@@ -261,18 +277,19 @@ class Cluster():
         Instance method to display the disks usage in human readable
         """
         print("Disk usage human readable output:")
-        for ring,usage in self.usage_human.items():
+        for ring, usage in self.usage_human.items():
             print(f"  {ring}: {usage['used']} used on {usage['size']} {usage['percent']}%")
         print('')
 
     def display_filling(self):
         """
-        Instance method to display the disks filling in pseudo graph for object ring
+        Instance method to display the disks filling in pseudo graph
+        for object ring
         """
-        print(f"Disk filling for object ring:")
-        tpercent,tvalue = 0, 0
+        print("Disk filling for object ring:")
+        tpercent, tvalue = 0, 0
         # Calcul number of disks by representing by #
-        div = int(max( self.filling.values()) / 100) + 1
+        div = int(max(self.filling.values()) / 100) + 1
         for k, v in self.filling.items():
             # Shows disk fill distribution
             nb = round(v / div)
@@ -284,12 +301,12 @@ class Cluster():
         # Show disks with less than fmin padding
         if self.fmin and self.fmin.items():
             print(f"\nList of Disk filling < {self.args.disk.fmin} for object ring:")
-            for k, v in self.fmin.items():
+            for k, v in sorted(self.fmin.items(), key=lambda x:x[1]):
                 print(f"\t {k} -> {round(v,2)}%")
         # Show disks with more than fmax padding
         if self.fmax and self.fmax.items():
             print(f"\nList of Disk filling > {self.args.disk.fmax} for object ring:")
-            for k, v in self.fmax.items():
+            for k, v in sorted(self.fmax.items(), key=lambda x: x[1], reverse=True):
                 print(f"\t {k} -> {round(v,2)}%")
 
     def display_deviation(self):
@@ -313,11 +330,13 @@ class Cluster():
             "disk_usage": self.usage,
             "disk_usage_human": self.usage_human,
             "disk_filling": self.filling,
-            "df_min":self.fmin,
+            "df_min": self.fmin,
             "df_max": self.fmax,
             "deviation": self.deviation,
         }
         print(json.dumps(formated_output))
+
+
 class Ring():
     """Class representing a ring of  Cluster"""
 
@@ -344,18 +363,22 @@ class Ring():
             self.hosts[dev['ip']] = Host.create(dev)
             d[dev['ip']] = dev['port']
         self.host_port = set(d.items())
-    
+
     def load_devices(self):
         """
-        Instance method to load informations for all devices and assign to hosts
+        Instance method to load informations for all devices
+        and assign to hosts
         """
-        scout = recon.Scout(recon_type="diskusage", timeout=timeout, suppress_errors=True)
-        for url, response, status, _ts_start, _ts_end in pool.imap(scout.scout, self.host_port):
+        scout = recon.Scout(recon_type="diskusage",
+                            timeout=timeout,
+                            suppress_errors=True)
+        for url, response, status, _ts_start, _ts_end in \
+                pool.imap(scout.scout, self.host_port):
             ip = urlparse(url).hostname
             if status == 200:
-                port = urlparse(url).port
                 for res in response:
-                    self.hosts[ip].devices_by_name[res['device']].load_datas(res)
+                    self.hosts[ip].devices_by_name[
+                        res['device']].load_datas(res)
             else:
                 self.hosts[ip].error = True
 
@@ -367,7 +390,7 @@ class Ring():
         ip = urlparse(url).hostname
         try:
             hostname = socket.gethostbyaddr(ip)[0]
-        except:
+        except Exception:
             hostname = ip
         self.hosts_in_error.add(hostname)
 
@@ -377,10 +400,13 @@ class Ring():
         return: list of hosts with ntp drift
         """
         jitter = abs(0.0)
-        scout = recon.Scout(recon_type="time", timeout=timeout, suppress_errors=True)
+        scout = recon.Scout(recon_type="time",
+                            timeout=timeout,
+                            suppress_errors=True)
         ntpdrift_data = []
         swiftrecon._ptime
-        for url, ts_remote, status, ts_start, ts_end in pool.imap(scout.scout, self.host_port):
+        for url, ts_remote, status, ts_start, ts_end \
+                in pool.imap(scout.scout, self.host_port):
             if status != 200:
                 self.add_host_in_error(url)
                 continue
@@ -402,11 +428,14 @@ class Ring():
         """
         errors = []
         validate_data = []
-        scout = recon.Scout(recon_type="server_type_check", timeout=timeout, suppress_errors=True)
-        for url, response, status in pool.imap(scout.scout_server_type, self.host_port):
+        scout = recon.Scout(recon_type="server_type_check",
+                            timeout=timeout,
+                            suppress_errors=True)
+        for url, response, status in pool.imap(scout.scout_server_type,
+                                               self.host_port):
             if status == 200:
                 if response != self.ring_name + "-server":
-                        errors.append(url)
+                    errors.append(url)
             else:
                 self.add_host_in_error(url)
         for url in errors:
@@ -424,22 +453,22 @@ class Ring():
         size = 0
         for h in self.hosts.values():
             for d in h.devices_by_port.values():
-                used += d.used if isinstance(d.used,int) else 0
-                avail += d.avail if isinstance(d.avail,int) else 0
-                size += d.size if isinstance(d.size,int) else 0
+                used += d.used if isinstance(d.used, int) else 0
+                avail += d.avail if isinstance(d.avail, int) else 0
+                size += d.size if isinstance(d.size, int) else 0
         percent = round(100 * used / size, 2)
-        return {'used':used, 'avail':avail, 'size':size, 'percent':percent}
+        return {'used': used, 'avail': avail, 'size': size, 'percent': percent}
 
-    def disks_usage_human(self,base2):
+    def disks_usage_human(self, base2):
         """
         Instance method to load disk usage of all hosts
         return: dict with used, avail, total size and percent filling
         """
         du = self.disks_usage()
-        return {'used':size_suffix(du['used'],base2),
-                'avail':size_suffix(du['avail'],base2),
-                'size':size_suffix(du['size'],base2),
-                'percent':du['percent']}
+        return {'used': size_suffix(du['used'], base2),
+                'avail': size_suffix(du['avail'], base2),
+                'size': size_suffix(du['size'], base2),
+                'percent': du['percent']}
 
     def disks_filling(self):
         """
@@ -455,7 +484,7 @@ class Ring():
                     freq[d.filling] = 1
         freq = collections.OrderedDict(sorted(freq.items()))
         return freq
-    
+
     def disks_deviation(self):
         """
         Instance method to load deviation of filling
@@ -463,10 +492,10 @@ class Ring():
         """
         filling = []
         for h in self.hosts.values():
-            filling += [ d.filling for d in h.devices_by_port.values() ]
+            filling += [d.filling for d in h.devices_by_port.values()]
         return (std(filling))
 
-    def disks_fmin(self,dfmin):
+    def disks_fmin(self, dfmin):
         """
         Instance method to load devices filling less than dfmin
         param dfmin: the minimum filling to detect
@@ -479,11 +508,11 @@ class Ring():
                     usage = 100 * d.used / d.size
                     if usage < dfmin:
                         stats[f"{h.hostname}/{d.name}"] = usage
-                except:
+                except Exception:
                     pass
         return stats
 
-    def disks_fmax(self,dfmax):
+    def disks_fmax(self, dfmax):
         """
         Instance method to load devices filling more than dfmax
         param dfmax: the maximum filling to detect
@@ -496,7 +525,7 @@ class Ring():
                     usage = 100 * d.used / d.size
                     if usage > dfmax:
                         stats[f"{h.hostname}/{d.name}"] = usage
-                except:
+                except Exception:
                     pass
         return stats
 
@@ -524,9 +553,11 @@ class Ring():
                     unknown[h.region][h.hostname].append(d.name)
         return unknown
 
+
 class Host():
     """Class representing an host"""
     List = {}
+
     def create(device):
         """
         Class method to create a device and attach it to an host instance
@@ -594,17 +625,19 @@ class Device():
             self.unknown = True
         try:
             self.filling = round(100 * self.used / self.size)
-        except:
+        except Exception:
             pass
 
+
 def main():
     """
     Entry point for the script
     """
     args = _prepare_config()
     args(sys.argv[1:])
-    cluster = Cluster(ring_names,args)
+    cluster = Cluster(ring_names, args)
     cluster.display()
 
+
 if __name__ == "__main__":
     sys.exit(main())
